/*$Id: copy.c,v 1.4 1998/02/21 06:22:00 lindberg Exp $*/
/*$Name: ezmlm-idx-030 $*/

/* Copies a file relative the current directory and substitutes    */
/* !A at the beginning of a line for the target,                   */
/* !R at the beginning of a line for the confirm reply address,    */
/* the first <#A#> for target, the first <#R#> for confirm, and    */
/* the first <#L#> on a line for the list local name. Subsequent   */
/* tags on any line are ignored. */

#include "stralloc.h"
#include "substdio.h"
#include "strerr.h"
#include "str.h"
#include "getln.h"
#include "case.h"
#include "readwrite.h"
#include "qmail.h"
#include "errtxt.h"
#include "copy.h"
#include "mime.h"

static stralloc line = {0};
static stralloc outline = {0};
static stralloc qline = {0};
static substdio sstext;
static char textbuf[256];
static stralloc *outlocal = (stralloc *) 0;
static char *target = "?????";
static char *confirm = "?????";

void set_cpoutlocal(ln)
stralloc *ln;
{
  outlocal = ln;
}

void set_cptarget(tg)
char *tg;
{
  target = tg;
}

void set_cpconfirm(cf)
char *cf;
{
  confirm = cf;
}

static struct qmail *qq;

static void codeput(l,n,code,fatal)
char *l;
unsigned int n;
char code;
char *fatal;

{
  if (!code)
    qmail_put(qq,l,n);
  else {
    if (code == 'Q')
      encodeQ(l,n,&qline,fatal);
    else
      encodeB(l,n,&qline,0,fatal);
    qmail_put(qq,qline.s,qline.len);
  }
}

static void codeputs(l,code,fatal)
char *l;
char code;
char *fatal;
{
  codeput(l,str_len(l),code,fatal);
}

void copy(qqp,fn,q,fatal)
struct qmail *qqp;
char *fn;		/* text file name */
char q;			/* = '\0' for regular output, 'B' for base64, */
			/* 'Q' for quoted printable */
char *fatal;		/* FATAL error string */

{
  int fd;
  int match, done;
  unsigned int pos;

  qq = qqp;
  if ((fd = open_read(fn)) == -1)
    strerr_die4sys(111,fatal,ERR_OPEN,fn,": ");
  substdio_fdbuf(&sstext,read,fd,textbuf,sizeof(textbuf));
  for (;;) {
    if (getln(&sstext,&line,&match,'\n') == -1)
      strerr_die4sys(111,fatal,ERR_READ,fn,": ");
    if (match) {
      if (!stralloc_0(&line))
        strerr_die2x(111,fatal,ERR_NOMEM);
      if (line.s[0] == '!') {
	if (line.s[1] == 'R') {
	  codeput("   ",3,q,fatal);
	  codeputs(confirm,q,fatal);
	  codeput("\n",1,q,fatal);
	  continue;
	}
	if (line.s[1] == 'A') {
	  codeput("   ",3,q,fatal);
	  codeputs(target,q,fatal);
	  codeput("\n",1,q,fatal);
	  continue;
	}
      }
		/* Find the first tag <#x#> on a line. if x=A, replace  */
		/* with target, if x=R with recipient. Unrecognized tags*/
		/* are ignored. Only the first tag on a line is processed*/
      pos = 0;
      done = 0;
      while (!done && line.s[pos += str_chr(line.s+pos,'<')]) {
        if (line.s[pos+1] == '#' &&
            line.s[pos+2] &&
            line.s[pos+3] == '#' &&
            line.s[pos+4] == '>') {
          if (line.s[pos+2] == 'A') {
            if (!stralloc_copyb(&outline,line.s,pos)) die_nomem(fatal);
            if (!stralloc_cats(&outline,target)) die_nomem(fatal);
            if (!stralloc_catb(&outline,line.s+pos+5,line.len-pos-5))
			die_nomem(fatal);
            done = 1;
          } else if (line.s[pos+2] == 'R') {
            if (!stralloc_copyb(&outline,line.s,pos)) die_nomem(fatal);
            if (!stralloc_cats(&outline,confirm)) die_nomem(fatal);
            if (!stralloc_catb(&outline,line.s+pos+5,line.len-pos-5))
			die_nomem(fatal);
            done = 1;
          } else if (line.s[pos+2] == 'l') {
            if (!stralloc_copyb(&outline,line.s,pos)) die_nomem(fatal);
            if (!stralloc_catb(&outline,outlocal->s,outlocal->len))
			die_nomem(fatal);
            if (!stralloc_catb(&outline,line.s+pos+5,line.len-pos-5))
			die_nomem(fatal);
            done = 1;
          } else 
            pos += 5;	/* ignore unrecognized tag, but look for another */
        } else
          pos++;	/* not tag. Try from next char */
      }
      if (!done)
        codeput(line.s,line.len-1,q,fatal); 
      else
        codeput(outline.s,outline.len-1,q,fatal); 
    } else
      break;
  }
  close(fd);
}

