/*$Id: ezmlm-gate.c,v 1.6 1998/01/17 20:56:45 lindberg Exp $*/
/*$Name: ezmlm-idx-030 $*/

#include "stralloc.h"
#include "strerr.h"
#include "error.h"
#include "env.h"
#include "sig.h"
#include "str.h"
#include "fork.h"
#include "wait.h"
#include "exit.h"
#include "auto_bin.h"
#include "sgetopt.h"
#include "errtxt.h"
#include "idx.h"
#include "subscribe.h"

int flagcase = 1;	/* preserve sender case */

#define FATAL "ezmlm-gate: fatal: "

void die_usage()
{
  strerr_die1x(100,"ezmlm-gate: usage: ezmlm-gate [-vV] dir [moddir]");
}
void die_nomem() { strerr_die2x(111,FATAL,ERR_NOMEM); }

stralloc line = {0};
stralloc send = {0};
stralloc sendopt = {0};
stralloc storeopt = {0};

char szchar[2] = "-";

void main(argc,argv)
int argc;
char **argv;
{
  char *dir;
  char *sender;
  char *moddir;
  int child;
  char *sendargs[4];
  int wstat;
  char *pmod;
  int opt;

  umask(022);
  sig_pipeignore();

  if (!stralloc_copys(&sendopt," -")) die_nomem();
  if (!stralloc_copys(&storeopt," -")) die_nomem();

  while ((opt = getopt(argc,argv,
      "aAbBcCdDeEfFgGhHiIjJkKlLmMnNoOpPqQrRsStTuUvVxXyYzZ")) != opteof)
    switch(opt) {	/* pass on unrecognized options */
      case 'c':			/* ezmlm-send flags */
      case 'C': 
        szchar[0] = opt;
        if (!stralloc_append(&sendopt,szchar)) die_nomem();
        break;
      case 'v':
      case 'V': strerr_die1x(0,"ezmlm-gate version: $Name: ezmlm-idx-030 $\n");
      default:			/* ezmlm-store flags */
        szchar[0] = opt;
        if (!stralloc_append(&storeopt,szchar)) die_nomem();
    }

  dir = argv[optind++];
  if (!dir) die_usage();

  sender = env_get("SENDER");

  if (argv[optind])
    moddir = argv[optind++];
  else
    moddir = dir;
  pmod = 0;
  while (moddir && !pmod) {
    pmod = issub(moddir,sender,FATAL);
    moddir = argv[optind++];
  }

  sendargs[0] = "sh";
  sendargs[1] = "-c";
  if (!stralloc_copys(&send,auto_bin)) die_nomem();
  if (pmod) {
    if (!stralloc_cats(&send,"/ezmlm-send")) die_nomem();
    if (sendopt.len > 2)
      if (!stralloc_cat(&send,&sendopt)) die_nomem();
  } else {
    if (!stralloc_cats(&send,"/ezmlm-store")) die_nomem();
    if (storeopt.len > 2)
      if (!stralloc_cat(&send,&storeopt)) die_nomem();
  }
  if (!stralloc_cats(&send," '")) die_nomem(); 
  if (!stralloc_cats(&send,dir)) die_nomem();
  if (!stralloc_cats(&send,"'")) die_nomem();
  if (!stralloc_0(&send)) die_nomem();
  sendargs[2] = send.s;
  sendargs[3] = 0;

  switch(child = fork()) {
    case -1:
      strerr_die2sys(111,FATAL,ERR_FORK);
    case 0:
      execvp(*sendargs,sendargs);
      if (errno == error_txtbsy || errno == error_nomem ||
          errno == error_io)
        strerr_die4sys(111,FATAL,ERR_EXECUTE,sendargs[2],": ");
      else
        strerr_die4sys(100,FATAL,ERR_EXECUTE,sendargs[2],": ");
   }
         /* parent */
   wait_pid(&wstat,child);
   if (wait_crashed(wstat))
     strerr_die2x(111,FATAL,ERR_CHILD_CRASHED);
   switch(wait_exitcode(wstat)) {
     case 100:
       strerr_die2x(100,FATAL,"Fatal error from child");
     case 111:
        strerr_die2x(111,FATAL,"Temporary error from child");
     case 0:
       _exit(0);
     default:
       strerr_die2x(111,FATAL,"Unknown temporary error from child");
   }
}

