#include "stralloc.h"
#include "subfd.h"
#include "strerr.h"
#include "error.h"
#include "qmail.h"
#include "env.h"
#include "sig.h"
#include "open.h"
#include "getln.h"
#include "case.h"
#include "str.h"
#include "readwrite.h"
#include "exit.h"
#include "substdio.h"
#include "getconf.h"
#include "constmap.h"
#include "fmt.h"
#include "byte.h"
#include "errtxt.h"
#include "idx.h"

#define FATAL "ezmlm-request: fatal: "

void die_usage()
{
  strerr_die1x(100,"ezmlm-request: usage: ezmlm-request dir");
}

void die_nomem()
{
  strerr_die2x(111,FATAL,ERR_NOMEM);
}

void die_badaddr()
{
  strerr_die2x(100,FATAL,ERR_BAD_ADDRESS);
}

char strnum[FMT_ULONG];

stralloc line = {0};
stralloc subject = {0};
stralloc inlocal = {0};
stralloc inhost = {0};
stralloc outlocal = {0};
stralloc outhost = {0};
stralloc headerremove = {0};
struct constmap headerremovemap;

struct qmail qq;

char inbuf[1024];
substdio ssin = SUBSTDIO_FDBUF(read,0,inbuf,(int) sizeof(inbuf));

substdio ssout;
char outbuf[1];

stralloc mydtline = {0};

void main(argc,argv)
int argc;
char **argv;
{
  char *dir;
  char *host;
  char *local;
  char *sender;
  int flagmlwasthere;
  int match;
  int flaginheader;
  int flagbadfield;
  unsigned int pos,len;

  (void)umask(022);
  sig_pipeignore();

  dir = argv[1];
  if (!dir) die_usage();

  if (chdir(dir) == -1)
    strerr_die4sys(111,FATAL,ERR_SWITCH,dir,": ");

	/* do minimum to identify request for this program in case */
	/* it's invoked in line with e.g. ezmlm-manage */

  local = env_get("LOCAL");
  if (!local) strerr_die2x(100,FATAL,ERR_NOLOCAL);
  getconf_line(&inlocal,"inlocal",1,FATAL,dir);
  len = str_len(local);
  if (inlocal.len > len) die_badaddr();
  if (case_diffb(inlocal.s,inlocal.len,local)) die_badaddr();

  if ((inlocal.len + str_len(ACTION_REQUEST) != len) ||
      case_diffs(local+inlocal.len,ACTION_REQUEST))
    if ((inlocal.len + str_len(ALT_REQUEST) != len) ||
      case_diffs(local+inlocal.len,ALT_REQUEST))
    _exit(0);	/* not for us */

  sender = env_get("SENDER");
  if (!sender) strerr_die2x(100,FATAL,ERR_NOSENDER);
  host = env_get("HOST");
  if (!host) strerr_die2x(100,FATAL,ERR_NOHOST);
  getconf_line(&inhost,"inhost",1,FATAL,dir);
  if (inhost.len != str_len(host)) die_badaddr();
  if (case_diffb(inhost.s,inhost.len,host)) die_badaddr();

  if (!*sender)
    strerr_die2x(100,FATAL,ERR_BOUNCE);
  if (!sender[str_chr(sender,'@')])
    strerr_die2x(100,FATAL,ERR_ANONYMOUS);
  if (str_equal(sender,"#@[]"))
    strerr_die2x(100,FATAL,ERR_BOUNCE);

  getconf_line(&outhost,"outhost",1,FATAL,dir);
  getconf_line(&outlocal,"outlocal",1,FATAL,dir);

  getconf(&headerremove,"headerremove",1,FATAL,dir);
  constmap_init(&headerremovemap,headerremove.s,headerremove.len,0);

  if (!stralloc_copys(&mydtline,
       "Delivered-To: request processor for mailing list ")) die_nomem();
  if (!stralloc_catb(&mydtline,outlocal.s,outlocal.len)) die_nomem();
  if (!stralloc_cats(&mydtline,"@")) die_nomem();
  if (!stralloc_catb(&mydtline,outhost.s,outhost.len)) die_nomem();
  if (!stralloc_cats(&mydtline,"\n")) die_nomem();

  if (qmail_open(&qq) == -1)
    strerr_die2sys(111,FATAL,ERR_QMAIL_QUEUE);

  qmail_put(&qq,mydtline.s,mydtline.len);

  flagmlwasthere = 0;
  flaginheader = 1;
  flagbadfield = 0;

  for (;;) {
    if (getln(&ssin,&line,&match,'\n') == -1)
      strerr_die2sys(111,FATAL,ERR_READ_INPUT);

    if (flaginheader && match) {
      if (line.len == 1)
	flaginheader = 0;
      if ((line.s[0] != ' ') && (line.s[0] != '\t')) {
        flagbadfield = 0;
	if (constmap(&headerremovemap,line.s,byte_chr(line.s,line.len,':')))
	  flagbadfield = 1;
	else if (case_startb(line.s,line.len,"mailing-list:"))
	  flagmlwasthere = 1;
		/* no lwsp allowed, so no continuation lines possible */
	else if (case_startb(line.s,line.len,"Subject:")) {
          pos = 8;
          if (line.s[pos] == ' ' || line.s[pos] == '\t') pos++; 
		/* scrap terminal '\n' */
	  if (!stralloc_copyb(&subject,line.s+pos,line.len-pos-1)) die_nomem();
        } else if (line.len == mydtline.len)
	  if (!byte_diff(line.s,line.len,mydtline.s))
            strerr_die2x(100,FATAL,ERR_LOOPING);
      }
    }
    if (!(flaginheader && flagbadfield))
      qmail_put(&qq,line.s,line.len);
    if (!match)
      break;
  }
  if (flagmlwasthere)
    strerr_die2x(100,FATAL,ERR_MAILING_LIST);

  qmail_from(&qq,sender);

  if (subject.len > inlocal.len && 
       !case_diffb(subject.s,inlocal.len,inlocal.s) &&
       subject.s[inlocal.len] == '-') {
    pos = byte_chr(subject.s,subject.len,'@');
    if (!stralloc_copy(&line,&outlocal)) die_nomem();
    if (!stralloc_catb(&line,subject.s+inlocal.len,pos-inlocal.len))
               die_nomem();
    if (!stralloc_cats(&line,"@")) die_nomem();
    if (!stralloc_cat(&line,&outhost)) die_nomem();
    if (!stralloc_0(&line)) die_nomem();
    qmail_to(&qq,line.s);
  } else {
    len = byte_chr(subject.s,subject.len,' ');
    if (!stralloc_copy(&line,&outlocal)) die_nomem();
    if (!stralloc_cats(&line,"-")) die_nomem();
    if (!stralloc_catb(&line,subject.s,len)) die_nomem();
    if (!stralloc_cats(&line,"@")) die_nomem();
    if (!stralloc_cat(&line,&outhost)) die_nomem();
    if (!stralloc_0(&line)) die_nomem();
    qmail_to(&qq,line.s);
  }
  switch(qmail_close(&qq)) {
    case 0:
      strnum[fmt_ulong(strnum,qmail_qp(&qq))] = 0;
      strerr_die2x(99,"ezmlm-request: info: qp ",strnum);
    default:
      strerr_die2x(111,FATAL,ERR_TMP_QMAIL_QUEUE);
  }
}
