/*$Id: idx.h,v 1.29 1998/03/24 22:08:25 lindberg Exp $*/
/*$Name: ezmlm-idx-030 $*/
#ifndef IDX_H
#define IDX_H

/* the multipart type of the digest. Pine has a bug in the handling of */
/* explicitly typed parts of multipart. If you use multipart/mixed, it */
/* won't be able to display multipart messages that are part of a digest*/
/* and if you use multipart/digest, it won't be able to display the digest*/
/* TOC when content-transfer-encoding is used. May also be important for*/
/* other (broken) MUAs. Best is to use Mutt, which is happy with either */

#define DIGEST_MIME "Content-Type: multipart/mixed"

/* Range for '-thread' to protect large archives. A '-thread' search */
/* will start at most THREAD_BEFORE messages before the action argument */
/* and go to at most THREAD_AFTER messages after the action argument. */
#define THREAD_BEFORE 2000
#define THREAD_AFTER 2000

/* Maximum number of messages returned by get */
/* You also have to update /text/bottom in ezmlmrc if you change this */
#define MAXGET 100

/* Maximum subject index entries returned by index */
/* Must be multiple of 100 */
/* You also have to update /text/bottom in ezmlmrc if you change this */
#define MAXINDEX 2000

/* Max dir/text file size allowed by -edit */
#define MAXEDIT 10240

/* Timeout in seconds before a bounce warning is sent. Default is  */
/* 1000000, i.e. 11.57 days. Setting it lower reduces the number of */
/* messages in the bouce dir, but makes it more likely that an address */
/* is unsubscribed due to a temporary error. This compile-time default */
/* should rarely need changing, as it can be overridden with the ezmlm-warn */
/* -t switch */
#define BOUNCE_TIMEOUT 1000000L

/* Command names and alternative command names */
/* all alternates must be defined! */
/* The language-specific blocks need to undefine and redefine commands */

#define ALT_LIST "-list"
#define ALT_EDIT "-edit"
#define ALT_GET "-get"
#define ALT_HELP "-help"
#define ALT_INDEX "-index"
#define ALT_THREAD "-thread"
#define ALT_REQUEST "-request"
#define ALT_SUBSCRIBE "-subscribe"
#define ALT_UNSUBSCRIBE "-unsubscribe"

/* to get alternative command names, you need to undefine and redefine */
/* them. Do this within a language block and send it to the author for */
/* inclusing in future versions. If it's here already, just uncommend the */
/* define for the appropriate language. */
/* #define LANG_FR 1 */

/* French Version */
#ifdef LANG_FR
#undef ALT_SUBSCRIBE
#define ALT_SUBSCRIBE "-inscription"
#undef ALT_UNSUBSCRIBE
#define ALT_UNSUBSCRIBE "-desinscription"
#endif
/* end French Version */

/* Text that is used in the outgoing messages (there is some other text, but */
/* it needs to stay constant in order to comply with rfc1153 */

/* Topics (messages nnn through mmm):\n */
#define TXT_TOP_TOPICS "Topics"
#define TXT_TOP_MESSAGES " (messages "
#define TXT_TOP_THROUGH " through "
#define TXT_TOP_LAST "):\n"

/* in digest */
#define TXT_ADMINISTRIVIA "\nAdministrivia:\n\n"
#define TXT_SUPPRESSED "\n<suppressed>\n\n"

/* for the message author line: 000 by */
/* keep this short! */
#define TXT_BY " by: "

/* Since this is now run-time configurable, we'll go with the lowest */
/* common denominator (per rfc2046). -> ISO-8859-1 if you don't like that */
#define TXT_DEF_CHARSET "us-ascii"

/* should start with 20 'a' [in place of hash] */
#define TXT_NOINDEX "aaaaaaaaaaaaaaaaaaaa <- subject index not available for message(s) ->\n"

/* When copy of the message is suppressed (is this really used?)*/
#define TXT_SUPPRESSED "\n<suppressed>\n\n"

/* Subject: MODERATE for inlocal@inhost */
#define TXT_MODERATE "MODERATE for "

/* Subject: Returned post for inlocal@inhost */
/* (used both for rejected and timed-out posts) */
#define TXT_RETURNED_POST "Returned post for "

/* Subject: CONFIRM subscribe to | unsubscribe from */
#define TXT_USRCONFIRM "confirm "
#define TXT_MODCONFIRM "CONFIRM "
#define TXT_SUBSCRIBE_TO "subscribe to "
#define TXT_UNSUBSCRIBE_FROM "unsubscribe from "

/* Subject: WELCOME to */
#define TXT_WELCOME "Subject: WELCOME to "

/* Subject: GOODBYE from */
#define TXT_GOODBYE "Subject: GOODBYE from "

/* Subject: ezmlm response\n\n */
#define TXT_EZMLM_RESPONSE "Subject: ezmlm response\n\n"

/* Subject: Edit file xxx for list@host */
#define TXT_EDIT_RESPONSE "Subject: EDIT "
#define TXT_EDIT_FOR " for "

/* Subject: Editable text files\n\n */
#define TXT_EDIT_LIST "Subject: List of editable text files\n\n"

/* markers for ezmlm-manage text file edit */
/* MUST start with '%' */
#define TXT_EDIT_START "%%% START OF TEXT FILE"
#define TXT_EDIT_END "%%% END OF TEXT FILE"

#define TXT_EDIT_SUCCESS "Subject: Success editing "

/* Text for '-list' command */
#define TXT_LISTMEMBERS "\nSubscribers to this list are:\n\n"

/* Output formats - letter used to override default */
#define FORMATS "mrvn"
#define MIME 'm'
#define RFC1153 'r'
/* ---------------- virgin = MIME without header processing */
#define VIRGIN 'v'
/* NATIVE 'n' = VIRGIN without threading */
#define NATIVE 'n'
/* default output format. */
#define DEFAULT_FORMAT MIME

/* Use MIME enclosure for message to moderate by default (1) or not (0) */
/* ezmlm-store switches -m/-M override */
#define MOD_MIME 1

/* Mode of messages in archive. For ezmlm-0.53 this is 0744, but for  */
/* "secret" lists it may make more sense to make it 0700.             */
#define MODE_ARCHIVE 0744

/* ezmlm-get actions  (ACTION_GET also for -get in ezmlm-manage) */
#define ACTION_GET "-get"
#define ACTION_INDEX "-index"
#define ACTION_THREAD "-thread"

/* ezmlm-request actions */
#define ACTION_REQUEST "-request"

/* actions for post acceptance/rejection */
#define ACTION_ACCEPT "-accept-"
#define ACTION_REJECT "-reject-"

/* ezmlm-manage actions */
#define ACTION_LIST "-list"
#define ACTION_HELP "-help"
#define ACTION_SUBSCRIBE "-subscribe"
#define ACTION_UNSUBSCRIBE "-unsubscribe"
#define ACTION_EDIT "-edit"
#define ACTION_ED "-ed."
#define LENGTH_ED 4

/* ACTION_XC has to be a string "-xc." where x is any letter. All commands */
/* should have different letters. They no longer have to match the first    */
/* letter of subscribe/unsubscribe. */
/* The third char of ACTION_SC/TC/UV/VC has to be 'c' */

/* user subscription confirm */
#define ACTION_SC "-sc."
/* moderator subscription confirm */
#define ACTION_TC "-tc."
/* user unsubscribe confirm */
#define ACTION_UC "-uc."
/* moderator unsubscribe confirm */
#define ACTION_VC "-vc."

/* name addition for digest, i.e. list-"digest" Don't change! */
#define ACTION_DIGEST "-digest"

/* defaults for message time out in moderation queue. If modsub is 0 */
/* or empty, DELAY_DEFAULT is used. If it is set, it is made to be  */
/* within DELAY_MIN .. DELAY_MAX. All in hours. */
#define DELAY_MIN 24
#define DELAY_DEFAULT 120
#define DELAY_MAX 240

/* Mode of messages in moderation queue. The owner mode is |'d with 7.*/
/* The group/world mode can be set to anything, but it really doesn't */
/* make sense to make these messages visible to anyone else.          */
#define MODE_MOD_MSG 0700

/* name and location of system-wide customized ezmlmrc. This is where */
/* ezmlm-make looks first (unless the -c switch is specified) before  */
/* falling back to the (usually unchanged) version in the ezmlm bin   */
/* directory. */
#define TXT_ETC_EZMLMRC "/etc/ezmlmrc"

/* name of config file for ezmlm-cron */
#define TXT_EZCRONRC "ezcronrc"

/*---------- Things below this line are not configurable -----------*/
/* Action types */
#define AC_GET 0
#define AC_DIGEST 1
#define AC_THREAD 2
#define AC_INDEX 3

typedef struct msgentry {	/* one per message in range */
  unsigned long subnum;		/* subject number */
  char *author;			/* message author */
} msgentry;

typedef struct subentry {	/* one per unique subject in message range */
  void *higher;
  void *lower;
  char *sub;			/* string with terminating '\0' */
				/* when building, higher/lower=0 marks end */
				/* of branch. When printing, start at the  */
				/* beginning of the table and go up until  */
				/* sub = 0. */
  unsigned long firstmsg;	/* the first message with this subject*/
} subentry;

#endif

