/*$Id: */
/*$Name: */

#include "stralloc.h"
#include "strerr.h"
#include "case.h"
#include "byte.h"
#include "errtxt.h"

/* Characters */
#define ESC '\x1B'
#define SI '\x0F'
#define SO '\x0E'
/* iso-2022-jp back-to-ascii seq */
#define TOASCII "\x1B(B"
/* iso-2022 SI sequence as string */
#define TOSI "\x0F"

int trimre(cpp,cpend,prefix,fatal)
char **cpp;
char *cpend;
stralloc *prefix;
char *fatal;

{
  int r = 0;
  register char *cp;
  char *cpnew;
  int junk;
  unsigned int serial;

  cp = *cpp;
  serial = prefix->len;		/* pointer to serial number */
  if (serial)
    serial = byte_chr(prefix->s,prefix->len,'#');
  junk = 1;
  while (junk) {
    junk = 0;
    while (cp <= cpend && (*cp == ' ' || *cp == '\t')) cp++;
    if (cp + 2 <= cpend)		/* at least 'Re:' */
      if (case_startb(cp,2,"re") || case_startb(cp,2,"aw") ||
          case_startb(cp,3,"ang")) {
        cpnew = cp + 1;
        if (*cpnew == 'n') ++cpnew;	/* "Ang:" */
        while (++cpnew <= cpend) {
          register char ch;
          ch = *cpnew;
          if (ch == ':') {		/* terminal ':' always required */
            junk = 1;
            r |= 1;
            cp = cpnew + 1;
            break;
          }				/* NOTE: no letters, no ' ' allowed */
          if (ch >= '0' && ch <= '9') continue;
          if (ch =='[' || ch ==']' || ch == '^') continue;
          if (ch == '(' || ch == ')') continue;
        }
    }
    if (prefix->len) {
      if (serial == prefix->len) {
        if((cp + prefix->len <= cpend) &&
               case_startb(cp,prefix->len,prefix->s)) {
          cp += prefix->len;
          junk = 1;
          r |= 2;
        }
      } else {				/* prefix with serial number */
        if((cp + prefix->len <= cpend) && !case_diffb(cp,serial,prefix->s)) {
          cpnew = cp + serial;		/* point to serial number */
          while (cpnew <= cpend && *cpnew >= '0' && *cpnew <= '9') ++cpnew;
          if ((cpnew + prefix->len - serial - 1 <= cpend) &&
		!case_diffb(cpnew,prefix->len-serial-1,prefix->s+serial+1)) {
            cp = cpnew + prefix->len - serial - 1;
            junk = 1;
            r |= 2;
          }
        }
      }
    }
  }
  *cpp = cp;
  return r;
}

int trimend(indata,n,fatal)
char *indata;
unsigned int *n;
char *fatal;
	/* looks at indata of length n from the end removing LWSP & '\n' */
	/* and any trailing '-Reply'. Sets n to new length and returns:  */
	/* 0 - not reply, 1 - reply. */
{
  char *cplast;
  int junk;
  int r = 0;

  if (*n == 0) return 0;
  cplast = indata + *n - 1;	/* points to last char on line */
  junk = 1;
  while (junk) {
    junk = 0;
    while (cplast >= indata &&
             (*cplast == ' ' || *cplast == '\t' ||
              *cplast == '\r' || *cplast == '\n')) 
            --cplast;
    if (cplast - indata  >= 5 && case_startb(cplast - 5,6,"-Reply")) {
      cplast -= 6;
      r = 1;
      junk = 1;
    }
  }
  *n = (unsigned int) (cplast - indata + 1);	/* new length */
  return r;
}

int unfoldHDR(indata,n,outdata,charset,prefix,flagtrimsub,fatal)
char *indata;
unsigned int n;
stralloc *outdata;
char *charset;
stralloc *prefix;
int flagtrimsub;
char *fatal;
	/* takes a header as indata. Removal of reply-indicators is done */
	/* but removal of line breaks and Q and B decoding should have   */
	/* been done. Returns a */
	/* single line header without trailing \n or \0. Mainly, we      */
	/* remove redundant shift codes   */
	/* returns 0 = no reply no prefix */
	/*         1 = reply no prefix    */
	/*         2 = no reply, prefix   */
	/*         3 = reply & pefix      */
{
  int r = 0;
  char *cp,*cpesc,*cpnext,*cpend,*cpskip;
  char state,cset,newcset;
  int reg,newreg;

  cp = indata;		/* JIS X 0201 -> ISO646 us-ascii */
  cpend = cp + n - 1;
  cpnext = cp;
  if (!stralloc_copys(outdata,"")) die_nomem(fatal);
  if (!stralloc_ready(outdata,n)) die_nomem(fatal);

  if(!case_diffb(charset,11,"iso-2022-jp")) {
	/* iso-2022-jp-2 (rfc1554) and its subset iso-2022-jp. The reg #s */
	/* are from the rfc. Don't ask why they have multiple length G0   */
	/* charset designations ... JIS X 0201-roman is identical to      */
	/* iso646 us-ascii except for currency and tilde. Making them the */
	/* same increases hits without significant loss. JIS X 0208-1978  */
	/* is superceded by JIS X 0208-1983 and converted here as well.   */

    while (cp <= cpend) {
      if (*cp++ != ESC) continue;
      if (*cp == '(') {
        ++cp;
        if (*cp == 'J') *cp = 'B';
        ++cp;
      } else if (*cp == '$') {
        ++cp; 
        if (*cp == '@') *cp = 'B';
        ++cp;
      }
    }
		/* eliminate redundant ESC seqs */
    cp = indata;
    cpnext = cp;
    reg = 6;
    while (cp <= cpend) {
      if (*cp++ != ESC) continue;
      cpesc = cp - 1;
      if (*cp == '$') {
        ++cp;
        if (*cp == 'B') newreg = 87;
        else if (*cp == 'A') newreg = 58;
        else if (*cp == '(') {
          ++cp;
          if (*cp == 'C') newreg = 149;
          else if (*cp == 'D') newreg = 159;
          else continue;
        } else continue;
      } else if (*cp == '(') {
        ++cp;
        if (*cp == 'B') newreg = 6;
        else continue;
      } else continue;
      ++cp;
      while (*cp == ' ' || *cp == '\t') ++cp;	/* skip space */
      if (*cp == ESC)			/* maybe another G0 designation */
        if (*(cp+1) == '(' || *(cp+1) == '$') {	 /* yep! */
          if (!stralloc_catb(outdata,cpnext,cpesc-cpnext)) die_nomem(fatal);
          cpnext = cp;
	  continue;
      }
      if (reg == newreg) {
        if (!stralloc_catb(outdata,cpnext,cpesc-cpnext)) die_nomem(fatal);
        cpnext = cp;
      } else {
        if (flagtrimsub) {	/* first state change. Scrap leading Re:...*/
          r |= trimre(&cpnext,cpesc,prefix,fatal);
          flagtrimsub = 0;
        }
        reg = newreg;
      }		/* copy remainder of line */
    }
    if (flagtrimsub) {	/* No state change yet - scrap leading Re:...*/
      r |= trimre(&cpnext,cpend,prefix,fatal);
      flagtrimsub = 0;
    }
    if (!stralloc_catb(outdata,cpnext,cpend - cpnext + 1)) die_nomem(fatal);
    if (reg != 6) {	/* need to return to us-ascii at the end of the line */
      if (!stralloc_cats(outdata,TOASCII)) die_nomem(fatal);
    } else {		/* maybe "-Reply at the end?" */
      r |= trimend(outdata->s,&(outdata->len),fatal);
    }

  } else if (!case_diffb(charset,11,"iso-2022-cn") ||
             !case_diffb(charset,11,"iso-2022-kr")) {
	/* these use SI/SO and ESC $ ) x as the SO designation. In -cn and */
	/* -cn-ext, 'x' can be a number of different letters. In -kr it's  */
	/* always 'C'. This routine may work also for other iso-2022 sets  */
	/* also handles iso-2022-cn-ext */
    cpesc = (char *) 0;	/* points to latest ESC */
    state = SI;		/* us-ascii */
	/* Re: can be before G0 designator or after G0 designator. If there*/
	/* is any text before the G0 designator, we don't need to look after*/
    if (flagtrimsub)		/* do stuff before */
      r = trimre(&cpnext,cpend,prefix,fatal);
    while (cp <= cpend) {
      if (*cp == SI || *cp == SO) {
        if (flagtrimsub) {
          flagtrimsub = 0;
          if (cpesc) {	/* have passed a G0 designation */
            cpskip = cpesc+4;
            if (cpskip < cp) {
              r |= trimre(&cpskip,cp,prefix,fatal);
              if (cpskip != cpesc + 4) {
                if (!stralloc_catb(outdata,cpnext,cpesc-cpnext+4))
			 die_nomem(fatal);
                cpnext = cpskip;
              }
            }
          }
        }
        if (state == *cp) {
          if (!stralloc_catb(outdata,cpnext,cp-cpnext-1)) die_nomem(fatal);
          ++cp;
          cpnext = cp;
        } else
          state == *cp;
        ++cp;
        continue;
      }
      if (*cp++ != ESC) continue;
      cpesc = cp - 1;
      if (*cp != '$') continue;
      ++cp;
      if (*cp != ')') continue;
      ++cp;
      newcset = *cp;
      ++cp;
      while (*cp == ' ' || *cp == '\t') ++cp;	/* skip space */
      if (*cp == ESC && *(cp+1) == '$' && *(cp+2) == ')') {
        if (!stralloc_catb(outdata,cpnext,cpesc-cpnext)) die_nomem(fatal);
        continue;
      } else if (newcset == cset) {
        if (!stralloc_catb(outdata,cpnext,cpesc-cpnext)) die_nomem(fatal);
      } else {
        cset = newcset;
        continue;
      }
    }
    if (!stralloc_catb(outdata,cpnext,cpend - cpnext + 1)) die_nomem(fatal);
    if (state != SI)
      if (!stralloc_cats(outdata,TOSI)) die_nomem(fatal);
    else		/* maybe "-Reply at the end?" */
      r |= trimend(outdata->s,&(outdata->len),fatal);

  } else {
    if (flagtrimsub) {		 /* remove leading reply indicators & prefix*/
      r = trimre(&cp,cpend,prefix,fatal);
      n = (unsigned int) (cpend-cp+1);
    }
    r |= trimend(cp,&n,fatal);
    if (!stralloc_catb(outdata,cp,n)) die_nomem(fatal);
  }
		/* there shouldn't be '\0' or '\n', but make sure as */
		/* it would break the message index */
  cp = outdata->s;
  cpend = cp + outdata->len;
  while (cp < cpend) {
   if (!*cp || *cp == '\n') *cp = ' ';
   ++cp;
  }
  return r;
}

